

//
// RocksSuperScanner is NOT a typical scanner.  Its intent is to catch
// cheat-vehicles, and in this regard, the scanner is MUCH MORE than a
// typical vehicle-scanner.  It also verifies validity of some vehicle
// component and weapons parameters.  Note that not all parameters can 
// be verified by the server, but it does verify component/weapon sizes
// in relation to their mount-points.
//
// This scanner works differently than other scanners.  It automatically
// scans vehicles when they drop on the field and attaches the scan results
// to the vehicle object as a string.  Then, when a player scans his
// targeted vehicle, it simply dumps the vehicle scan-string that was
// originally generated when the vehicle first dropped onto the field.
// This little trick cuts down on lag because it doesn't do scanning in
// real-time.  The side effect is that it presents the info of the original,
// fully functional vehicle rather than the immediate battle-condition of
// the vehicle.  The fully functional information is usually the info that
// everyone wants anyway.
//
// I measured the elapsed time of the scanning of an Apoc with maximum
// components loaded.  Despite all the work that the scanner is apparently
// doing, the total elapsed time was beyond the measurement resolution of
// the timing function, which is down to a millisecond.  This means that
// actual scan time is insignificant... something in the microseconds
// region on a 1 Ghz P3.  Rest assured that when a vehicle drops onto the
// field, the scan activity won't be contributing anything to server-lag.
//
//						Rock D.
//


// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------- SuperScanner Entry Points --------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// This is the info-dumper of the SuperScanner.  The sayPlayer is the
// player wanting the scan information.  This relies on the fact that
// the vehicle has already been pre-scanned at vehicle spawn time and
// this routine just dumps the scanString.
//
// This is also the Admin's entry point into the SuperScanner if the
// RocksSuperScan variable has been configured to true.  A player's entry
// is in the EXTVehicleOnScan() routine below, which only allows it if
// the RocksPlayerScan variable is true.
//
function RocksSuperScanner(%sayPlayer, %vehicle)
{
	if (!%vehicle)
		return;

	say(%sayPlayer, 0, %vehicle.scanString);
}


// --------------------------------------------------------------------------
// This is the automatic entry point from vehicleOnAdd().  It activates
// every time a vehicle is spawned.  Skips AI vehicles.
//
function EXTVehicleOnAdd(%vehicle)
{
	if (!$RocksSuperScan)
		return;

	%player = playerManager::vehicleIdToPlayerNum(%vehicle);

	if (%player) {
		%vehicle.scanString = "";
		schedule("RocksAutoScanner (" @ %vehicle @ ");", 0.4);
	}
}


// --------------------------------------------------------------------------
// This is the normal, in-game, Scan Key (I-Key) entry point as long as
// the RocksPlayerScan variable has been configured to allow it.
//
function EXTVehicleOnScan(%scanned, %scanner)
{
	if (!$RocksPlayerScan)
		return;

	%sayPlayer = playerManager::vehicleIdToPlayerNum(%scanner);

	schedule("RocksSuperScanner (" @
		%sayPlayer @ ", " @ %scanned @ ");", 0.2);
}




// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------- Scanning Engine ------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// ------ Everything below here is internal to the scanning engine. ---------
// -----------  Use the above entry points to access it. --------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
					// Sensors
$CompName[400] = "Basic Sensor";
$CompSize[400] = 0;
$CompMass[400] = 1.0;

$CompName[401] = "Ranger Sensor";
$CompSize[401] = 0;
$CompMass[401] = 2.5;

$CompName[408] = "Stnd Sensor";
$CompSize[408] = 0;
$CompMass[408] = 2.0;

$CompName[409] = "LBow Sensor";
$CompSize[409] = 1;
$CompMass[409] = 2.5;

$CompName[410] = "Infil Sensor";
$CompSize[410] = 1;
$CompMass[410] = 3.5;

$CompName[411] = "Crossbow Sensor";
$CompSize[411] = 2;
$CompMass[411] = 3.0;

$CompName[412] = "UltraLt Sensor";
$CompSize[412] = 0;
$CompMass[412] = 0.5;

$CompName[413] = "Bldhnd Sensor";
$CompSize[413] = 1;
$CompMass[413] = 1.5;

$CompName[414] = "EmScan Sensor";
$CompSize[414] = 1;
$CompMass[414] = 1.5;

$CompName[426] = "CyBasic Sensor";
$CompSize[426] = 0;
$CompMass[426] = 1.0;

$CompName[427] = "CyEnh Sensor";
$CompSize[427] = 0;
$CompMass[427] = 2.5;

$CompName[428] = "CyInter Sensor";
$CompSize[428] = 0;
$CompMass[428] = 2.0;

$CompName[429] = "CyLRnge Sensor";
$CompSize[429] = 1;
$CompMass[429] = 2.5;

$CompName[430] = "CyPassv Sensor";
$CompSize[430] = 1;
$CompMass[430] = 3.0;

$CompName[431] = "CyExt Sensor";
$CompSize[431] = 2;
$CompMass[431] = 2.5;

$CompName[432] = "CyCmpct Sensor";
$CompSize[432] = 0;
$CompMass[432] = 0.5;

$CompName[433] = "CyTackr Sensor";
$CompSize[433] = 1;
$CompMass[433] = 1.5;

$CompName[434] = "CyMotn Sensor";
$CompSize[434] = 1;
$CompMass[434] = 1.5;

					// Reactors
$CompName[200] = "Micro Reactor";
$CompSize[200] = 0;
$CompMass[200] = 3.0;

$CompName[201] = "Small Reactor";
$CompSize[201] = 0;
$CompMass[201] = 4.0;

$CompName[202] = "Stnd Reactor";
$CompSize[202] = 0;
$CompMass[202] = 4.5;

$CompName[203] = "Medm Reactor";
$CompSize[203] = 1;
$CompMass[203] = 5.5;

$CompName[204] = "Large Reactor";
$CompSize[204] = 1;
$CompMass[204] = 6.5;

$CompName[205] = "Maxim Reactor";
$CompSize[205] = 2;
$CompMass[205] = 10.0;

$CompName[225] = "Alpha Reactor";
$CompSize[225] = 0;
$CompMass[225] = 3.0;

$CompName[226] = "Beta Reactor";
$CompSize[226] = 0;
$CompMass[226] = 3.5;

$CompName[227] = "Gamma Reactor";
$CompSize[227] = 0;
$CompMass[227] = 4.0;

$CompName[228] = "Delta Reactor";
$CompSize[228] = 1;
$CompMass[228] = 5.5;

$CompName[229] = "Epsln Reactor";
$CompSize[229] = 1;
$CompMass[229] = 6.0;

$CompName[230] = "Zeta Reactor";
$CompSize[230] = 2;
$CompMass[230] = 8.0;

						// Shields
$CompName[300] = "Stnd Shield";
$CompSize[300] = 0;
$CompMass[300] = 1.5;

$CompName[301] = "Prtctr Shield";
$CompSize[301] = 0;
$CompMass[301] = 2.0;

$CompName[302] = "Guardn Shield";
$CompSize[302] = 0;
$CompMass[302] = 2.5;

$CompName[303] = "FstChrg Shield";
$CompSize[303] = 1;
$CompMass[303] = 4.0;

$CompName[304] = "Centrn Shield";
$CompSize[304] = 1;
$CompMass[304] = 3.0;

$CompName[305] = "Replsr Shield";
$CompSize[305] = 2;
$CompMass[305] = 5.0;

$CompName[306] = "Titan Shield";
$CompSize[306] = 1;
$CompMass[306] = 3.5;

$CompName[307] = "Medusa Shield";
$CompSize[307] = 2;
$CompMass[307] = 8.5;

$CompName[326] = "Alpha Shield";
$CompSize[326] = 0;
$CompMass[326] = 1.5;

$CompName[327] = "Beta Shield";
$CompSize[327] = 0;
$CompMass[327] = 2.5;

$CompName[328] = "Gamma Shield";
$CompSize[328] = 0;
$CompMass[328] = 3.5;

$CompName[329] = "Delta Shield";
$CompSize[329] = 1;
$CompMass[329] = 4.0;

$CompName[330] = "Epsilon Shield";
$CompSize[330] = 1;
$CompMass[330] = 5.5;

$CompName[331] = "Zeta Shield";
$CompSize[331] = 2;
$CompMass[331] = 7.0;

$CompName[332] = "Eta Shield";
$CompSize[332] = 1;
$CompMass[332] = 6.5;

$CompName[333] = "Theta Shield";
$CompSize[333] = 2;
$CompMass[333] = 9.0;

						// Engines
$CompName[100] = "Lt Engine";
$CompSize[100] = 0;
$CompMass[100] = 1.5;

$CompName[101] = "LtHO Engine";
$CompSize[101] = 0;
$CompMass[101] = 2.0;

$CompName[102] = "LtAg Engine";
$CompSize[102] = 0;
$CompMass[102] = 2.5;

$CompName[103] = "StdMd Engine";
$CompSize[103] = 1;
$CompMass[103] = 3.0;

$CompName[104] = "MedHO Engine";
$CompSize[104] = 1;
$CompMass[104] = 3.5;

$CompName[105] = "MedAg Engine";
$CompSize[105] = 1;
$CompMass[105] = 4.0;

$CompName[106] = "StdHvy Engine";
$CompSize[106] = 2;
$CompMass[106] = 4.5;

$CompName[107] = "ImpHvy Engine";
$CompSize[107] = 2;
$CompMass[107] = 5.0;

$CompName[108] = "HvyCru Engine";
$CompSize[108] = 2;
$CompMass[108] = 5.5;

$CompName[109] = "HvyHO Engine";
$CompSize[109] = 2;
$CompMass[109] = 6.0;

$CompName[110] = "HvyAg Engine";
$CompSize[110] = 2;
$CompMass[110] = 7.0;

$CompName[111] = "StdAslt Engine";
$CompSize[111] = 3;
$CompMass[111] = 8.0;

$CompName[112] = "ImpAslt Engine";
$CompSize[112] = 3;
$CompMass[112] = 9.0;

$CompName[113] = "HvyTrbo Engine";
$CompSize[113] = 3;
$CompMass[113] = 10.0;

$CompName[114] = "HOT Engine";
$CompSize[114] = 3;
$CompMass[114] = 11.0;

$CompName[115] = "SupHvy Engine";
$CompSize[115] = 3;
$CompMass[115] = 12.5;

$CompName[128] = "AlphLt Engine";
$CompSize[128] = 0;
$CompMass[128] = 1.5;

$CompName[129] = "BetLtAg Engine";
$CompSize[129] = 0;
$CompMass[129] = 2.0;

$CompName[130] = "GamStdMed Engine";
$CompSize[130] = 1;
$CompMass[130] = 2.5;

$CompName[131] = "DltaMedCru Engine";
$CompSize[131] = 1;
$CompMass[131] = 3.0;

$CompName[132] = "EpsImpMed Engine";
$CompSize[132] = 1;
$CompMass[132] = 3.5;

$CompName[133] = "ZetaMedHO Engine";
$CompSize[133] = 1;
$CompMass[133] = 4.0;

$CompName[134] = "EtaMedAg Engine";
$CompSize[134] = 1;
$CompMass[134] = 4.5;

$CompName[135] = "ThtaStdHvy Engine";
$CompSize[135] = 2;
$CompMass[135] = 5.0;

$CompName[136] = "IotaHvyHO Engine";
$CompSize[136] = 2;
$CompMass[136] = 5.5;

$CompName[137] = "KappaHvyAg Engine";
$CompSize[137] = 2;
$CompMass[137] = 6.0;

$CompName[138] = "LamdaStd Engine";
$CompSize[138] = 3;
$CompMass[138] = 6.0;

$CompName[139] = "MuImp Engine";
$CompSize[139] = 3;
$CompMass[139] = 7.5;

$CompName[140] = "NuHO Engine";
$CompSize[140] = 3;
$CompMass[140] = 9.0;

$CompName[141] = "XiHvy Engine";
$CompSize[141] = 3;
$CompMass[141] = 10.0;

$CompName[142] = "OmiHvy Turbine";
$CompSize[142] = 3;
$CompMass[142] = 10.5;

$CompName[143] = "PiSupHvy Turbine";
$CompSize[143] = 3;
$CompMass[143] = 12.5;

						// Computers
$CompName[800] = "Basic Computer";
$CompSize[800] = 0;
$CompMass[800] = 1.0;

$CompName[801] = "Imprv Computer";
$CompSize[801] = 0;
$CompMass[801] = 1.5;

$CompName[802] = "Advcd Computer";
$CompSize[802] = 0;
$CompMass[802] = 2.0;

$CompName[805] = "CyBas Computer";
$CompSize[805] = 0;
$CompMass[805] = 0.5;

$CompName[806] = "CyEnh Computer";
$CompSize[806] = 0;
$CompMass[806] = 1.0;

$CompName[807] = "CyAdv Computer";
$CompSize[807] = 0;
$CompMass[807] = 1.5;

						// Specials
$CompName[810] = "Guardian Jammer";
$CompSize[810] = 0;
$CompMass[810] = 1.0;

$CompName[811] = "Dopple Jammer";
$CompSize[811] = 2;
$CompMass[811] = 2.0;

$CompName[812] = "CyAlpha Jammer";
$CompSize[812] = 0;
$CompMass[812] = 1.5;

$CompName[813] = "CyBeta Jammer";
$CompSize[813] = 2;
$CompMass[813] = 3.0;

$CompName[820] = "ThermDiff Jammer";
$CompSize[820] = 0;
$CompMass[820] = 2.0;

$CompName[830] = "Chameleon Cloak";
$CompSize[830] = 1;
$CompMass[830] = 1.0;

$CompName[831] = "Cuttlefish Cloak";
$CompSize[831] = 2;
$CompMass[831] = 3.5;

$CompName[840] = "SMOD";
$CompSize[840] = 0;
$CompMass[840] = 3.0;

$CompName[845] = "SCAP";
$CompSize[845] = 0;
$CompMass[845] = 1.0;

$CompName[850] = "SAMP";
$CompSize[850] = 1;
$CompMass[850] = 2.0;

$CompName[860] = "LTD ";
$CompSize[860] = 0;
$CompMass[860] = 1.0;

$CompName[865] = "AuxPower";
$CompSize[865] = 0;
$CompMass[865] = 1.0;

$CompName[870] = "EnergyCap";
$CompSize[870] = 0;
$CompMass[870] = 1.0;

$CompName[875] = "FieldStab";
$CompSize[875] = 0;
$CompMass[875] = 1.0;

$CompName[880] = "Rocket";
$CompSize[880] = 1;
$CompMass[880] = 5.0;

$CompName[885] = "Turbine";
$CompSize[885] = 0;
$CompMass[885] = 2.0;

$CompName[890] = "NanoRepair";
$CompSize[890] = 2;
$CompMass[890] = 6.0;

$CompName[900] = "AngelLife";
$CompSize[900] = 0;
$CompMass[900] = 0.5;

$CompName[910] = "AntiGrav";
$CompSize[910] = 2;
$CompMass[910] = -5.0;

$CompName[912] = "ElectroHull";
$CompSize[912] = 2;
$CompMass[912] = 5.0;

$CompName[914] = "UAP ";
$CompSize[914] = 1;
$CompMass[914] = 4.0;

						// Armor
$CompName[926] = "CarLam Armor";
$CompSize[926] = 0;
$CompMass[926] = 0.03;

$CompName[927] = "QBM Armor   ";
$CompSize[927] = 0;
$CompMass[927] = 0.05;

$CompName[928] = "Durac Armor ";
$CompSize[928] = 1;
$CompMass[928] = 0.08;

$CompName[929] = "Cerc Armor  ";
$CompSize[929] = 0;
$CompMass[929] = 0.06;

$CompName[930] = "Crys Armor  ";
$CompSize[930] = 1;
$CompMass[930] = 0.07;

$CompName[931] = "Quick Armor ";
$CompSize[931] = 2;
$CompMass[931] = 0.09;

						// Weapons
$WeapName[101] = "Las";
$WeapSize[101] = 0;
$WeapMass[101] = 1.0;

$WeapName[102] = "HLas";
$WeapSize[102] = 1;
$WeapMass[102] = 3.0;

$WeapName[103] = "CLas";
$WeapSize[103] = 0;
$WeapMass[103] = 2.5;

$WeapName[104] = "TLas";
$WeapSize[104] = 0;
$WeapMass[104] = 2.0;

$WeapName[105] = "EMP";
$WeapSize[105] = 0;
$WeapMass[105] = 1.5;

$WeapName[106] = "ELF";
$WeapSize[106] = 1;
$WeapMass[106] = 5.0;

$WeapName[107] = "Blstr";
$WeapSize[107] = 1;
$WeapMass[107] = 4.0;

$WeapName[108] = "HBlstr";
$WeapSize[108] = 2;
$WeapMass[108] = 5.0;

$WeapName[109] = "PBW";
$WeapSize[109] = 2;
$WeapMass[109] = 7.5;

$WeapName[110] = "Plasma";
$WeapSize[110] = 2;
$WeapMass[110] = 8.5;

$WeapName[111] = "Blink";
$WeapSize[111] = 1;
$WeapMass[111] = 3.5;

$WeapName[112] = "QGun";
$WeapSize[112] = 3;
$WeapMass[112] = 8.5;

$WeapName[113] = "MFAC";
$WeapSize[113] = 3;
$WeapMass[113] = 10.5;

$WeapName[114] = "NanInf";
$WeapSize[114] = 0;
$WeapMass[114] = 2.5;

$WeapName[115] = "NanCan";
$WeapSize[115] = 1;
$WeapMass[115] = 4.0;

$WeapName[116] = "ATC";
$WeapSize[116] = 0;
$WeapMass[116] = 1.5;

$WeapName[117] = "HATC";
$WeapSize[117] = 1;
$WeapMass[117] = 3.0;

$WeapName[118] = "EMC";
$WeapSize[118] = 0;
$WeapMass[118] = 2.0;

$WeapName[119] = "BC";
$WeapSize[119] = 2;
$WeapMass[119] = 8.0;

$WeapName[120] = "HBC";
$WeapSize[120] = 3;
$WeapMass[120] = 10.0;

$WeapName[121] = "Rail";
$WeapSize[121] = 2;
$WeapMass[121] = 6.0;

$WeapName[124] = "Vipr8";
$WeapSize[124] = 0;
$WeapMass[124] = 3.0;

$WeapName[125] = "Vipr12";
$WeapSize[125] = 2;
$WeapMass[125] = 4.0;

$WeapName[126] = "Spar6";
$WeapSize[126] = 0;
$WeapMass[126] = 2.0;

$WeapName[127] = "Spar10";
$WeapSize[127] = 1;
$WeapMass[127] = 3.0;

$WeapName[128] = "Swarm6";
$WeapSize[128] = 1;
$WeapMass[128] = 4.0;

$WeapName[129] = "Minion10";
$WeapSize[129] = 1;
$WeapMass[129] = 3.5;

$WeapName[130] = "Shrike8";
$WeapSize[130] = 1;
$WeapMass[130] = 5.0;

$WeapName[147] = "Aphid10";
$WeapSize[147] = 0;
$WeapMass[147] = 3.5;

$WeapName[131] = "Arach6";
$WeapSize[131] = 0;
$WeapMass[131] = 2.0;

$WeapName[132] = "Arach10";
$WeapSize[132] = 1;
$WeapMass[132] = 4.0;

$WeapName[133] = "Arach14";
$WeapSize[133] = 2;
$WeapMass[133] = 6.0;

$WeapName[134] = "Mine6";
$WeapSize[134] = 0;
$WeapMass[134] = 1.0;

$WeapName[135] = "Mine8";
$WeapSize[135] = 1;
$WeapMass[135] = 3.0;

$WeapName[136] = "Mine10";
$WeapSize[136] = 2;
$WeapMass[136] = 5.0;

$WeapName[142] = "RadGun";
$WeapSize[142] = 1;
$WeapMass[142] = 3.0;

$WeapName[3]   = "DisGun";
$WeapSize[3]   = 1;
$WeapMass[3]   = 4.0;

$WeapName[4]   = "EHull";
$WeapSize[4]   = 1;
$WeapMass[4]   = 5.0;

$WeapName[150] = "SGun";
$WeapSize[150] = 1;
$WeapMass[150] = 5.0;

						// Hercs
$HercName[1]	= "Apocalypse";
$HercMinMass[1] = 15.0;
$HercMaxMass[1] = 60.0;
$HercHrdPnts[1] = "MMXXSS";
$HercIntMnts[1] = "LMSLMLML";			// E R C S A S 1 2

$HercName[2]	= "Minotaur";
$HercMinMass[2] =  7.0;
$HercMaxMass[2] = 40.0;
$HercHrdPnts[2] = "SSMM";
$HercIntMnts[2] = "MMSMMMML";

$HercName[3]	= "Gorgon";
$HercMinMass[3] = 25.0;
$HercMaxMass[3] = 80.0;
$HercHrdPnts[3] = "XXLL";
$HercIntMnts[3] = "XLSLMMLL";

$HercName[4]	= "Talon";
$HercMinMass[4] =  6.0;
$HercMaxMass[4] = 30.0;
$HercHrdPnts[4] = "MM";
$HercIntMnts[4] = "SMSSMLLS";

$HercName[5]	= "Basilisk";
$HercMinMass[5] = 10.0;
$HercMaxMass[5] = 50.0;
$HercHrdPnts[5] = "MMXX";
$HercIntMnts[5] = "LMSMMMML";

$HercName[10]	 = "Knight's Apocalypse";
$HercMinMass[10] = 13.0;
$HercMaxMass[10] = 60.0;
$HercHrdPnts[10] = "MMXXSS";
$HercIntMnts[10] = "LMSLMLLL";

$HercName[11]	 = "Knight's Minotaur";
$HercMinMass[11] =  6.0;
$HercMaxMass[11] = 40.0;
$HercHrdPnts[11] = "SSMM";
$HercIntMnts[11] =  "MMSMMMML";

$HercName[12]	 = "Knight's Gorgon";
$HercMinMass[12] = 22.0;
$HercMaxMass[12] = 80.0;
$HercHrdPnts[12] = "XXLL";
$HercIntMnts[12] = "XLSLMMLL";

$HercName[13]	 = "Knight's Talon";
$HercMinMass[13] =  5.0;
$HercMaxMass[13] = 30.0;
$HercHrdPnts[13] = "MMLL";
$HercIntMnts[13] = "SMSMMLLS";

$HercName[14]	 = "Knight's Basilisk";
$HercMinMass[14] =  8.0;
$HercMaxMass[14] = 50.0;
$HercHrdPnts[14] = "MMXX";
$HercIntMnts[14] = "LMSLMSML";

$HercName[20]	 = "Seeker";
$HercMinMass[20] =  4.0;
$HercMaxMass[20] = 25.0;
$HercHrdPnts[20] = "MM";
$HercIntMnts[20] = "SMSMMLLS";

$HercName[21]	 = "Goad";
$HercMinMass[21] =  8.0;
$HercMaxMass[21] = 35.0;
$HercHrdPnts[21] = "MM";
$HercIntMnts[21] = "MMSMMMLS";

$HercName[22]	 = "Shepherd";
$HercMinMass[22] =  8.0;
$HercMaxMass[22] = 48.0;
$HercHrdPnts[22] = "MMMM";
$HercIntMnts[22] = "LMSLMMML";

$HercName[23]	 = "Adjudicator";
$HercMinMass[23] = 12.0;
$HercMaxMass[23] = 60.0;
$HercHrdPnts[23] = "MMXX";
$HercIntMnts[23] = "XMSLMSML";

$HercName[24]	 = "Executioner";
$HercMinMass[24] = 25.0;
$HercMaxMass[24] = 90.0;
$HercHrdPnts[24] = "XXLL";
$HercIntMnts[24] = "XLSLMLLL";

$HercName[30]	 = "Emancipator";
$HercMinMass[30] = 10.0;
$HercMaxMass[30] = 40.0;
$HercHrdPnts[30] = "XM";
$HercIntMnts[30] = "MLSMMMML";

$HercName[33]	 = "Olympian";
$HercMinMass[33] = 15.0;
$HercMaxMass[33] = 75.0;
$HercHrdPnts[33] = "XXLLSS";
$HercIntMnts[33] = "XLSLMLLL";

$HercName[35]	 = "Metagen Seeker";
$HercMinMass[35] =  4.0;
$HercMaxMass[35] = 25.0;
$HercHrdPnts[35] = "MM";
$HercIntMnts[35] = "SMSMMLLS";

$HercName[36]	 = "Metagen Goad";
$HercMinMass[36] =  8.0;
$HercMaxMass[36] = 35.0;
$HercHrdPnts[36] = "MM";
$HercIntMnts[36] = "MMSMMMLS";

$HercName[37]	 = "Metagen Shepherd";
$HercMinMass[37] =  8.0;
$HercMaxMass[37] = 48.0;
$HercHrdPnts[37] = "MMMM";
$HercIntMnts[37] = "LMSLMMML";

$HercName[52]	 = "Outrider Emancipator";
$HercMinMass[52] = 10.0;
$HercMaxMass[52] = 40.0;
$HercHrdPnts[52] = "XM";
$HercIntMnts[52] = "MLSMMMML";

$HercName[55]	 = "Platinum Guard Adjudicator";
$HercMinMass[55] = 10.0;
$HercMaxMass[55] = 60.0;
$HercHrdPnts[55] = "MMXX";
$HercIntMnts[55] = "XLSLMSML";

$HercName[56]	 = "Platinum Guard Executioner";
$HercMinMass[56] = 23.0;
$HercMaxMass[56] = 90.0;
$HercHrdPnts[56] = "XXLL";
$HercIntMnts[56] = "XLSLMLLL";

						// Tanks
$HercName[6]	= "Paladin";
$HercMinMass[6] = 15.0;
$HercMaxMass[6] = 60.0;
$HercHrdPnts[6] = "LL";
$HercIntMnts[6] = "LMSLLMLM";			// E R C A S 1 2 3

$HercName[7]	= "Myrmidon";
$HercMinMass[7] = 40.0;
$HercMaxMass[7] = 90.0;
$HercHrdPnts[7] = "XX";
$HercIntMnts[7] = "XLSLMMLL";

$HercName[8]	= "Disrupter";
$HercMinMass[8] = 20.0;
$HercMaxMass[8] = 55.0;
$HercHrdPnts[8] = "S";
$HercIntMnts[8] = "LLSLSMLL";

$HercName[15]	 = "Knight's Paladin";
$HercMinMass[15] = 12.0;
$HercMaxMass[15] = 60.0;
$HercHrdPnts[15] = "LL";
$HercIntMnts[15] = "LMSLLMLM";

$HercName[16]	 = "Knight's Myrmidon";
$HercMinMass[16] = 36.0;
$HercMaxMass[16] = 90.0;
$HercHrdPnts[16] = "XX";
$HercIntMnts[16] = "XLSLMMLL";

$HercName[17]	 = "Knight's Disruptor";
$HercMinMass[17] = 18.0;
$HercMaxMass[17] = 55.0;
$HercHrdPnts[17] = "L";
$HercIntMnts[17] = "LLSLSMLM";

$HercName[25]	 = "Bolo";
$HercMinMass[25] =  7.0;
$HercMaxMass[25] = 35.0;
$HercHrdPnts[25] = "MM";
$HercIntMnts[25] = "MLSLSSLM";

$HercName[26]	 = "Recluse";
$HercMinMass[26] =  8.0;
$HercMaxMass[26] = 40.0;
$HercHrdPnts[26] = "MLL";
$HercIntMnts[26] = "MMSLSMML";

$HercName[31]	 = "Avenger";
$HercMinMass[31] =  3.0;
$HercMaxMass[31] = 30.0;
$HercHrdPnts[31] = "LL";
$HercIntMnts[31] = "SLSLSMLM";

$HercName[32]	 = "Dreadlock";
$HercMinMass[32] = 14.0;
$HercMaxMass[32] = 45.0;
$HercHrdPnts[32] = "XX";
$HercIntMnts[32] = "LLSLSMLM";

$HercName[41]	 = "Harabec's Predator";
$HercMinMass[41] = 18.0;
$HercMaxMass[41] = 60.0;
$HercHrdPnts[41] = "LL";
$HercIntMnts[41] = "LLSLMMLL";



// --------------------------------------------------------------------------
// This is the wrapper interface routine for the Scanning Engine.
// This routine is scheduled from the EXTvehicleOnAdd() function every
// time someone enters the field with a new vehicle.  Any failed vehicle
// is automatically logged to a file, assuming the filename variable is set.
//
function RocksAutoScanner(%vehicle)
{
	if (!RocksScanEngine(0, %vehicle)) {

		%player = playerManager::vehicleIdToPlayerNum(%vehicle);

		// The Admin Package uses this to trigger an auto-ban
		//
		%player.cheat = 1;

		if ($RocksScanLogFilename == "")
			return(0);


		fileWrite($RocksScanLogFilename, append, "");

		fileWrite($RocksScanLogFilename, append,
					"-------" @ RocksInfoMaker(%player));

		fileWrite($RocksScanLogFilename, append, %vehicle.scanString);

		return(0);
	}

	return(1);
}


// --------------------------------------------------------------------------
// Returns the nth character of a string, where n is normalized to 0.
// A space is returned as an underscore "_" character.  Returns 0 if the
// function failed.
// This is more of a character-manipulation library routine, but the
// SuperScanner is the only thing that uses it right now.
//
function RocksCharN(%str, %n)
{
	if (%n >= strlen(%str))
		return(0);

	%str = convertSpaces(%str);


	// get everything BUT the nth char
	//
	%sub = strAlign(%n, left, %str);


	// Now get everything including the nth char and tack
	// on a pseudo-char.  This fixes the math-character issues.
	//
	%sample = strAlign(%n+1, left, %str) @ "#";


	// find the mystery char
	//
	for (%j = 0; %j < 26; %j++) {
		if ((%sub @ $upper[%j] @ "#") == %sample)
			return($upper[%j]);

		if ((%sub @ $lower[%j] @ "#") == %sample)
			return($lower[%j]);
	}

	for (%j = 0; %j < 256; %j++) {
		if ((%sub @ $ascii[%j] @ "#") == %sample)
			return($ascii[%j]);
	}

	return(0);
}


// --------------------------------------------------------------------------
// Returns 1 if it's a Herc and 0 if it's a Tank
//
function RocksVehicleHerc(%vNum)
{
	if (((%vNum >=  6) && (%vNum <=  8)) ||
	    ((%vNum >= 15) && (%vNum <= 17)) ||
	    ((%vNum >= 25) && (%vNum <= 26)) ||
	    ((%vNum >= 31) && (%vNum <= 32)) || (%vNum == 41))
		return(0);

	return(1);
}


// --------------------------------------------------------------------------
// Given a list of hardpoints (slotList) and the Nth slot in the list, this
// routine verifies that the itemSize will fit into the slot
//
function RocksMountCheck(%slotList, %slotN, %itemsize)
{
	%c = RocksCharN(%slotList, %slotN);

	if (%c == "S")
		%slotSize = 0;
	else if (%c == "M")
		%slotSize = 1;
	else if (%c == "L")
		%slotSize = 2;
	else if (%c == "X")
		%slotSize = 3;

	if (%itemsize > %slotSize)
		return(1);

	if (%itemsize == %slotSize)
		return(0);

	return(-1);
}


// --------------------------------------------------------------------------
// Returns 1 if a component fits in its slot, else returns 0.
//
function RocksComponentCheck(%vNum, %compID)
{
	// Engine
	if ((%compID >= 100) && (%compID <= 143))
		return (RocksMountCheck($HercIntMnts[%vNum],
				0, $CompSize[%compID]) <= 0);

	// Reactor
	if ((%compID >= 200) && (%compID <= 230))
		return (RocksMountCheck($HercIntMnts[%vNum],
				1, $CompSize[%compID]) <= 0);

	// Computer
	if ((%compID >= 800) && (%compID <= 807))
		return (RocksMountCheck($HercIntMnts[%vNum],
				2, $CompSize[%compID]) <= 0);


	if (RocksVehicleHerc(%vNum)) {

		// Shields
		if ((%compID >= 300) && (%compID <= 333))
			return (RocksMountCheck($HercIntMnts[%vNum],
					3, $CompSize[%compID]) <= 0);

		// Armor
		if ((%compID >= 926) && (%compID <= 931))
			return (RocksMountCheck($HercIntMnts[%vNum],
					4, $CompSize[%compID]) <= 0);

		// Sensor
		if ((%compID >= 400) && (%compID <= 434))
			return (RocksMountCheck($HercIntMnts[%vNum],
					5, $CompSize[%compID]) <= 0);
	}
	else {
		// Armor
		if ((%compID >= 926) && (%compID <= 931))
			return (RocksMountCheck($HercIntMnts[%vNum],
					3, $CompSize[%compID]) <= 0);

		// Sensor
		if ((%compID >= 400) && (%compID <= 434))
			return (RocksMountCheck($HercIntMnts[%vNum],
					4, $CompSize[%compID]) <= 0);
	}
}


// --------------------------------------------------------------------------
// Given a list of Weapons or Special Components (cList) and a list of internal
// mounts or weapons hardpoints (slotList) and the Nth slot to begin with,
// this routine sorts the weaps/component list largest to smallest, then
// verifies that they can all fit into the available slots.
// Returns 1 if everything fits, else returns 0.
//
function RocksMountList(%slotList, %NthSlot, %cList)
{
	// Initialize the slot-consumption record
	//
	%slotlen = strlen(%slotList);

	for (%i = 0; %i < %slotlen; %i++)
		%checked[%i] = 0;


	// Itemize the cList
	//
	%len = strlen(%cList);

	for (%i = 0; %i < %len; %i++)
		%sorted[%i] = RocksCharN(%cList, %i);


	// Now sort the cList largest to smallest
	//
	for (%i = 0; %i < %len-1; %i++) {

		%done = 1;

		for (%j = 0; %j < %len-1; %j++) {
			if (%sorted[%j] < %sorted[%j+1]) {
				%save = %sorted[%j];
				%sorted[%j] = %sorted[%j+1];
				%sorted[%j+1] = %save;
				%done = 0;
			}
		}

		if (%done)
			break;
	}


	// For each Special, see if it fits into a slot and record that
	// the slot is consumed.  If all Specials fit, then all is well.
	//
	for (%i = 0; %i < %len; %i++) {
		for (%j = %NthSlot; %j < %slotlen; %j++) {
			if (%checked[%j])
				continue;

			if (RocksMountCheck(%slotList, %j, %sorted[%i]) <= 0) {
				%checked[%j] = 1;
				break;
			}
		}

		if (%j == %slotlen)
			return(0);
	}

	return(1);
}


// --------------------------------------------------------------------------
// Assembles the string for column-display at a later time.  I think there's
// a better way to do this with chat formatting sequences.  But I didn't find
// that out until AFTER I had coded this up and started using it.  Now, I'm
// too lazy to change it.  I'll leave it as an exercise for the reader.
//
function RocksColShow(%accumulator, %col, %str)
{
	if ((%col % 3) < 0.5 )
		%accumulator = strcat(%accumulator, "\t");


	// This bumps the fields out so they all line up
	//
	if ((%col % 3) < 1.5 ) {
		for (%i = strlen(%str); %i < 20; %i += 4)
			%str = strcat(%str, "\t");
	}

	%accumulator = strcat(%accumulator, %str);

	if ((%col % 3) > 1.5)
		%accumulator = strcat(%accumulator, "\n");

	return(%accumulator);
}


// --------------------------------------------------------------------------
// This is the vehicle-scanning engine.  This is not called directly, but
// instead it's called through one of the wrapper-functions up above.  This
// scanner does more than just the typical vehicle-scanner.  It also verifies
// that all of the components on the vehicle are valid for their hardpoints.
// I wrote this because a player once showed me, via the clientTree() function,
// that someone else was cheating with 2 AntiGrav units (a client-side hack).
// This scanner will catch that sort of hack automatically.  The sayPlayer
// variable is the playerID of the player getting the info.  If sayPlayer
// is 0 then no one gets the output and just the returned value of 1 or 0
// is used for a pass/fail indication.  The resulting scan-string is then
// attached directly to the vehicle for later reference on demand.
//
function RocksScanEngine(%sayPlayer, %vehicle)
{
	%name = getVehicleName(%vehicle);

	for (%vNum = 1; %vNum < 57; %vNum++) {
		if (%name == $HercName[%vNum])
			break;
	}

	if (%vNum == 57) {

		%vehicle.scanString = "Vehicle " @ %name @ " not found.";

		return (0);
	}

	%player = playerManager::vehicleIdToPlayerNum(%vehicle);


	%bigstr = "\n\t";
	if (!%player)
		%bigstr = %bigstr @ "AI Vehicle: \t\t" @ %name;
	else %bigstr = %bigstr @ getName(%player) @ ": \t\t" @ %name;
	%bigstr = %bigstr @ "\n";

	%mass = $HercMinMass[%vNum];


	%passed = 1;

	%col = 0;


	// Verify the components.  Most have reserved slots but the Specials
	// have to be handled differently because they can be in mixed slots.
	//
	%list = "";
	%spclCount = 0;
	%spclID[0] = 0;
	%count = getComponentCount(%vehicle);

	for (%i = 0; %i < %count; %i++) {
		%compID = getComponentId(%vehicle, %i);

		if ($CompName[%compID] == "") {

			%bigstr = RocksColShow(%bigstr, %col,
				"UKNOWN COMPONENT ID: " @ %compID);
			%col++;

			%passed = 0;

			continue;
		}
		else {
			// armor is a multiplier rather than a summation
			//
			if ((%compID >= 926) && (%compID <= 931))
				%mass += ($CompMass[%compID] *
					  $HercMaxMass[%vNum]);
			else %mass += $CompMass[%compID];


			// check for Disrupter specialties
			//
			if ((%compID == 912) &&
			    ((%vNum != 8) && (%vNum != 17))) {

				%bigstr = RocksColShow(%bigstr, %col,
					"ILLEGAL COMPONENT: Electrohull");
				%col++;

				%passed = 0;

				continue;
			}


			// Verify the component fits in it's hardpoint.
			// Specials are handled by a different routine
			// because they can be in different slots,
			// so we collect specials in a list.
			//
			if ((%compID >= 810) && (%compID <= 914)) {
				%list = %list @ $CompSize[%compID];
				%spclID[%spclCount] = %compID;
				%spclCount++;
			}
			else {
				if (!RocksComponentCheck(%vNum, %compID)) {

					%bigstr = RocksColShow(%bigstr, %col,
						"ILLEGAL SIZE: " @
						$CompName[%compID]);
					%col++;

					%passed = 0;

					continue;
				}
			}

			%bigstr = RocksColShow(%bigstr, %col,
				"-" @ $CompName[%compID]);
			%col++;
		}
	}


	// Verify the Specials list, if any
	//
	if (%list != "") {

		// Assure there's only one of each Special
		//
		for (%i = 0; %i < %spclCount; %i++) {
			for (%j = %i+1; %j < %spclCount; %j++) {
				if (%spclID[%i] == %spclID[%j]) {
					%bigstr = RocksColShow(%bigstr, %col,
							"ILLEGAL SPECIAL: " @
							$CompName[%i]);
					%col++;

					%passed = 0;

					break;
				}
			}
			if (%j != %spclCount)
				break;
		}


		// This if-statement verifies that there was only
		// one of each Special
		//
		if (%i == %spclCount) {

			if (RocksVehicleHerc(%vNum))
				%i = 6;
			else %i = 5;

			if (!RocksMountList($HercIntMnts[%vNum], %i, %list)) {

				%bigstr = RocksColShow(%bigstr, %col,
					"ILLEGAL COMPONENTS DETECTED");
				%col++;

				%passed = 0;
			}
		}
	}



	// Finally, verify the weapons.  These are done like the Specials
	// because they can be mixed in different slots.
	//
	%list = "";
	%wstr = "";
	%count = getWeaponCount(%vehicle);

	for (%i = 0; %i < %count; %i++) {
		%weapID = getWeaponId(%vehicle, %i);

		if ($WeapName[%weapID] == "") {

			%bigstr = RocksColShow(%bigstr, %col,
				"UNKNOWN WEAPON ID: " @ %weapID);
			%col++;

			%passed = 0;

			continue;
		}
		else {
			%mass += $WeapMass[%weapID];

			// check for Disrupter specialties
			//
			if ((%weapID == 3) &&
			    ((%vNum != 8) && (%vNum != 17))) {

				%bigstr = RocksColShow(%bigstr, %col,
					"ILLEGAL WEAPON: " @
					$WeapName[%weapID]);
				%col++;

				%passed = 0;

				continue;
			}

			%wstr = %wstr @ "-" @ $WeapName[%weapID] @ " ";

			%list = %list @ $WeapSize[%compID];
		}
	}


	// Verify the Weapons list, if any
	//
	if (%list != "") {
		if (!RocksMountList($HercHrdPnts[%vNum], 0, %list)) {

			%bigstr = RocksColShow(%bigstr, %col,
				"ILLEGAL WEAPONS DETECTED");
			%col++;

			%passed = 0;
		}
	}

	if ((%col % 3) > 0.5)
		%bigstr = strcat(%bigstr, "\n");

	%bigstr = strcat(%bigstr, "\t" @ %wstr @ "\n");


	// For display purposes, round the mass UP to the
	// nearest 0.01 tons to get rid of the floating-point slop.
	//
	%tons = floor(((%mass + 0.01) * 10));

	if (%tons % 10) {
		%tons = floor(%tons/10) @ "." @ (%tons % 10);
	}
	else %tons = floor(%tons/10);


	%bigstr = strcat(%bigstr, "\t" @ "Total Mass: " @
		%tons @ " tons of " @ $HercMaxMass[%vNum] @ " max\n");

	if (%mass > ($HercMaxMass[%vNum] + 0.05)) {
		%bigstr = strcat(%bigstr, "\t" @ "OVERMASS by " @
			(%mass - $HercMaxMass[%vNum]) @ "\n");

		%passed = 0;
	}

	%bigstr = %bigstr @ "\n";

	if (!%passed)
		%bigstr = %bigstr @ "\tILLEGAL VEHICLE";

	%vehicle.scanString = %bigstr;

	return(%passed);
}
